<?php

declare(strict_types=1);

namespace Drupal\codit_batch_operations\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\user\UserInterface;

/**
 * Defines the BatchOpLog entity.
 *
 * @ingroup batch_op_log
 *
 * @ContentEntityType(
 *   id = "batch_op_log",
 *   label = @Translation("Batch Operation Log"),
 *   label_plural = @Translation("Batch Operation Logs"),
 *   label_collection = @Translation("Batch Operation Log"),
 *   handlers = {
 *     "storage" = "\Drupal\Core\Entity\Sql\SqlContentEntityStorage",
 *     "view_builder" = "Drupal\codit_batch_operations\BatchOpLogViewBuilder",
 *     "views_data" = "Drupal\codit_batch_operations\Entity\BatchOpLogViewsData",
 *     "form" = {
 *       "delete" = "Drupal\codit_batch_operations\Form\BatchOpLogDeleteForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     },
 *     "access" = "Drupal\codit_batch_operations\BatchOpLogAccessControlHandler",
 *   },
 *   base_table = "batch_op_log",
 *   translatable = FALSE,
 *   admin_permission = "view codit batch operations log",
 *   entity_keys = {
 *     "id" = "id",
 *     "name" = "name",
 *     "uuid" = "uuid",
 *     "uid" = "user_id",
 *     "executor" = "executor",
 *     "created" = "created",
 *   },
 *   links = {
 *     "canonical" = "/admin/config/development/batch_operations/log/{batch_op_log}",
 *     "delete-form" = "/admin/config/development/batch_operations/log/{batch_op_log}/delete",
 *     "collection" = "/admin/config/development/batch_operations/log",
 *   },
 * )
 */
class BatchOpLog extends ContentEntityBase implements ContentEntityInterface, BatchOpLogInterface {

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  public $entityFieldManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  public $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $values, $entity_type, $bundle = FALSE, $translations = []) {
    parent::__construct($values, $entity_type, $bundle, $translations);
    // Can't use Dependency Injection on a @ContentEntityType.
    $this->entityTypeManager = \Drupal::service('entity_type.manager');
    $this->entityFieldManager = \Drupal::service('entity_field.manager');
  }

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage, array &$values): void {
    parent::preCreate($storage, $values);
    $values += [
      'user_id' => \Drupal::currentUser()->id(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function urlRouteParameters($rel): array {
    $uri_route_parameters = parent::urlRouteParameters($rel);
    return $uri_route_parameters;
  }

  /**
   * {@inheritdoc}
   */
  public function appendLog($message): BatchOpLogInterface {
    if ($message) {
      $existing_log = $this->get('log')->value;
      $this->set('log', $existing_log . PHP_EOL . $message);
      $this->setLastTime();
    }
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function appendError($message): BatchOpLogInterface {
    if ($message) {
      $existing_error = $this->get('errors')->value;
      $this->set('errors', $existing_error . PHP_EOL . $message);
      $this->setLastTime();
    }
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getErrors(): ?string {
    return $this->get('errors')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCompleted(): BatchOpLogInterface {
    $this->set('completed', TRUE);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getName(): string {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName(string $name): BatchOpLogInterface {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime(): int {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime(int $timestamp): BatchOpLogInterface {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setLastTime(): BatchOpLogInterface {
    $this->set('last', time());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('user_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('user_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('user_id', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('user_id', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getBatchOpLogScriptName($element): string {
    // This would return the name of the script, but might not be needed,
    // since likely it would be passed in.
    return $this->getName();
  }

  /**
   * {@inheritdoc}
   */
  public function getUri(): string {
    $uri = '/admin/config/development/batch_operations/log/' . $this->id();

    return $uri;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {
    $fields = parent::baseFieldDefinitions($entity_type);

    // Standard field, used as unique if primary index.
    $fields['id'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('ID'))
      ->setDescription(t('The ID of the BatchOpLog entity.'))
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => 0,
      ])
      ->setReadOnly(TRUE);

    // Standard field.
    $fields['uuid'] = BaseFieldDefinition::create('uuid')
      ->setLabel(t('UUID'))
      ->setDescription(t('The UUID of the BatchOpLog entity.'))
      ->setReadOnly(TRUE);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Script'))
      ->setDescription(t('The script that created this log.'))
      ->setRevisionable(FALSE)
      ->setSettings([
        'max_length' => 300,
        'text_processing' => 0,
      ])
      ->setDefaultValue('')
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => 2,
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', FALSE)
      ->setRevisionable(FALSE)
      ->setTranslatable(FALSE)
      // The script name is the unique identifier so require it.
      ->setRequired(TRUE);

    $fields['executor'] = BaseFieldDefinition::create('list_string')
      ->setLabel(new TranslatableMarkup('Executor'))
      ->setDescription(new TranslatableMarkup('What method ran the script.'))
      ->setTranslatable(FALSE)
      ->setSetting('allowed_values', [
        'ui' => new TranslatableMarkup('BatchOps UI'),
        'drush' => new TranslatableMarkup('Drush'),
        'hook_update' => 'hook_update_n',
        'post_update' => new TranslatableMarkup('Post update hook'),
        'deploy' => new TranslatableMarkup('Deploy hook'),
      ])
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => 4,
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', FALSE)
      ->setRequired(FALSE);

    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Run by'))
      ->setDescription(t('The user ID of user who ran this script.'))
      ->setRevisionable(FALSE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setTranslatable(FALSE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => 5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', FALSE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Started'))
      ->setDescription(t('The time that the Batch Ops Log was created / started.'))
      ->setDefaultValue(time())
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => 6,
      ]);

    $fields['last'] = BaseFieldDefinition::create('timestamp')
      ->setLabel(t('Last iteration'))
      ->setDescription(t('The time of the last log entry for the iteration, or the time the script completed.'))
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'weight' => 8,
      ]);

    $fields['completed'] = BaseFieldDefinition::create('boolean')
      ->setLabel(new TranslatableMarkup('Completed'))
      ->setDescription(t('If the script was run completely.'))
      ->setDisplayOptions('view', [
        'type' => 'boolean',
        'settings' => [
          'format' => 'yes-no',
        ],
        'label' => 'inline',
        'weight' => 9,
      ]);

    $fields['log'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Log'))
      ->setDescription(new TranslatableMarkup("The log of the script run."))
      ->setTranslatable(FALSE)
      ->setRevisionable(FALSE)
      ->setDefaultValue('')
      ->setDisplayOptions('view', [
        'type' => 'string',
        'weight' => 10,
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', FALSE)
      ->setRequired(FALSE);

    $fields['errors'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Errors'))
      ->setDescription(new TranslatableMarkup("Any caught errors."))
      ->setTranslatable(FALSE)
      ->setRevisionable(FALSE)
      ->setDefaultValue('')
      ->setDisplayOptions('view', [
        'type' => 'string',
        'weight' => 15,
      ])
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', FALSE)
      ->setRequired(FALSE);

    return $fields;
  }

}
